/**************************************************************************
****                        LCD_GEN_16X2.C                             ****
****                 Driver para modulo LCD comum de 16x2              ****
****                                                                   ****
****  Adaptao:Claudio Lrios                                         ****
****                                                                   ****
****                                                                   ****
****  Rotinas:                                                         ****
****                                                                   ****
****  lcd_init() Inicializao do LCD. Tem que ser chamado primeiro.   ****
****                                                                   ****
****  lcd_putc(c)  Ir mostrar 'c' na prxima posio do LCD.          ****
****                     Caracteres especiais:                         ****
****                      \f  Limpa tela                               ****
****                      \n  Vai para a primeira posio da 2 linha  ****
****                      \b  Volta uma posio para trs              ****
****                      \r  Vai para a primeira posio da 1 linha  ****
****                                                                   ****
****  lcd_gotoxy(x,y) Ajusta a posio de escrita no LCD               ****
****                                                                   ****
****  lcd_getc(x,y)   Retorna caracteres da posio x,y do LCD         ****
****                                                                   ****
**************************************************************************/


//#define use_lcd_pin_rw // comente se for ligar o rw no gnd.

#ifndef  lcd_pin_setup
//bits do lcd padro para portB (default)
#bit   db4       = 0x06.4 //rb4  db4 pino 10
#bit   db5       = 0x06.5 //rb1  db5 pino 11
#bit   db6       = 0x06.6 //rb2  db6 pino 12
#bit   db7       = 0x06.7 //rb3  db7 pino 13
#bit   en        = 0x06.0 //rb0  lcd 'enable' pino 6
#bit   rs        = 0x06.1 //rb1  lcd 'rs'  pino 7

#ifdef use_lcd_pin_rw
#bit   rw        = 0x06.2 //rb2  lcd "rw'  pino 8
#endif

#endif

#bit     rp0    = 0x03.5 //status,rp0
#define  bank1  rp0=1;
#define  bank0  rp0=0;


#define lcd_type 2           // 0=5x7, 1=5x10, 2=2 lines
#define lcd_line_two 0x40    // Endereo para 2 linha da RAM


BYTE const LCD_INIT_STRING[4] = {0x20 | (lcd_type << 2), 0xc, 1, 6};
                             // Bytes de configurao inicial do LCD

 #ifdef use_lcd_pin_rw //se for usado rw ligado ao microcontrolador e no ao gnd.
BYTE lcd_read_byte() {
      BYTE data;
      bank1
      db4=1; db5=1;  //ajusta os trisx de cada pino para leitura
      db6=1; db7=1;
      bank0
      rw = 1;
      delay_cycles(1);
      en = 1;
      delay_cycles(1);

      data=0;
      if(db4)data|=0x10;
      if(db5)data|=0x20;
      if(db6)data|=0x40;
      if(db7)data|=0x80;

      en = 0;
      delay_cycles(1);
      en = 1;
      delay_us(1);

      if(db4)data|=0x01;
      if(db5)data|=0x02;
      if(db6)data|=0x04;
      if(db7)data|=0x08;
      en = 0;
      bank1
      db4=0;db5=0;
      db6=0; db7=0;
      bank0
      rw=0;
      return(data);

}
#endif

void lcd_send_nibble( BYTE n ) {

      db4=0;db5=0;db6=0;db7=0;
      if(bit_test(n,0))db4=1;
      if(bit_test(n,1))db5=1;
      if(bit_test(n,2))db6=1;
      if(bit_test(n,3))db7=1;
      delay_cycles(1);
      en = 1;
      delay_us(2);
      en = 0;
}


void lcd_send_byte( BYTE address, BYTE d ) {

      rs = 0;
      #ifdef use_lcd_pin_rw
      while ( bit_test(lcd_read_byte(),7) ) ;
      #else
      delay_ms(4);
      #endif
      rs = address;
      delay_cycles(1);
      #ifdef use_lcd_pin_rw
      rw = 0;
      #endif
      delay_cycles(1);
      en = 0;
      lcd_send_nibble(d >> 4);
      lcd_send_nibble(d & 0xf);
}


void lcd_init() {
    BYTE i;
    bank1
    db4=0;db5=0;db6=0;db7=0;
    en=0;rs=0;
    #ifdef use_lcd_pin_rw
    rw=0;
    #endif
    bank0
    rs = 0;
    #ifdef use_lcd_pin_rw
    rw = 0;
    #endif
    en = 0;
    delay_ms(15);
    for(i=1;i<=3;++i) {
       lcd_send_nibble(3);
       delay_ms(5);
    }
    lcd_send_nibble(2);
    for(i=0;i<=3;++i)
       lcd_send_byte(0,LCD_INIT_STRING[i]);
}


void lcd_gotoxy( BYTE x, BYTE y) {
   BYTE address;

   if(y!=1)
     address=lcd_line_two;
   else
     address=0;
   address+=x-1;
   lcd_send_byte(0,0x80|address);
}

void lcd_putc( char c) {
   switch (c) {
     case '\f'   : lcd_send_byte(0,1);
                   delay_ms(2);
                                           break;
     case '\n'   : lcd_gotoxy(1,2);        break;
     case '\b'   : lcd_send_byte(0,0x10);  break;
     case '\r'   : lcd_gotoxy(1,1);        break;
     default     : lcd_send_byte(1,c);     break;
   }
}
#ifdef use_lcd_pin_rw
char lcd_getc( BYTE x, BYTE y) {
   char value;

    lcd_gotoxy(x,y);
    while ( bit_test(lcd_read_byte(),7) ); // espera at o flag de ocupado fique '0'
    rs=1;
    value = lcd_read_byte();
    rs=0;
    return(value);
}
#endif
